--> # altdoc
--> Alternative language-agnostic documentation generator using [MkDocs](https://github.com/mkdocs/mkdocs/)
--> and [highlight](http://www.andre-simon.de/doku/highlight/en/highlight.php).
import command                      from require "runkit"
argparse                               = require "argparse"
filekit                                = require "filekit"
--

sanitize = (str) -> str\gsub "[%(%)%.%%%+%-%*%?%[%]%^%$]", "%%%0"

generateMd = (text, prefix="-->") ->
  gl = (l) -> l\match "^#{sanitize prefix} ?(.+)"
  table.concat [gl line for line in text\gmatch "(.-)[\r\n]" when gl line], "\n"

import combine from filekit
linkSource = (text, file, source_dir="docs/source/", prefix="-->", anchor_prefix="--#") ->
  outfile = ((file\gsub "/", ".")..".html")
  lines   = [line for line in text\gmatch "(.-)[\r\n]"]
  for i=1, #lines do
    lines[i] = lines[i]\gsub "^#{sanitize anchor_prefix} ?(.+)",
      "#{prefix} *[source](#{"source/"..outfile.."#L_#{i}"})*<br>"
  table.concat lines, "\n"

formatSignatures = (text, prefix="-->", signature_prefix="--:") ->
  text\gsub "^#{sanitize signature_prefix} ?(.+)", "#{prefix} `%1`<br>"

generateSrc = (file, source_dir="/docs/source/", flags="-I -l --anchors -y L", extra_flags="") ->
  outfile = combine source_dir, ((file\gsub "/", ".")..".html")
  print (command "highlight -i \"$file\" -o \"$outfile\"") {:file, :outfile}, flags, extra_flags

import safeOpen from filekit
generateFile = (file, argt) ->
  generateSrc file, argt.source_dir, argt.highlight_flags, argt.highlight_extra
  --
  local contents
  with safeOpen file, "r"
    if .error then error .error
    contents = \read "*a"
    \close!
  text = (linkSource contents, file, argt.source_dir, argt.prefix, argt.anchor_prefix), argt.prefix
  text = formatSignatures text, argt.prefix, argt.signature_prefix
  md   = generateMd text, argt.prefix
  changext = (file\gsub "%.(.+)", ".md")\gsub "/", "."
  with safeOpen (combine argt.docs_dir, changext), "w"
    if .error then error .error
    \write md
    \close!

import makeDir, isDir, isFile, list from filekit
buildPathList = (path, total={}) ->
  if isDir path
    for node in *list path
      if (node == ".") or (node == "..") then continue
      buildPathList (combine path, node), total
  else
    table.insert total, path
  total

contains = (t, e) ->
  has = false
  for a in *t do has = true if a == e
  return has

--

local argl
with argparse!
  \name        "altdoc"
  \description "Language-agnostic documentation generator"
  \epilog      "Homepage - https://github.com/daelvn/altdoc"
  \help_description_margin 40
  \help_vertical_space 1

  with \argument "path", "File or folder to extract documentation from"
    \args 1

  with \option "-d --docs-dir", "Destination directory for the documentation"
    \args 1
    \default "docs/"

  with \option "-s --source-dir", "Destination directory for source files"
    \args 1
    \default "docs/source/"

  with \option "-p --prefix", "Comment prefix"
    \args 1
    \default "-->"

  with \option "-a --anchor-prefix", "Comment prefix for linking to source"
    \args 1
    \default "--#"

  with \option "-S --signature-prefix", "Comment prefix for signatures"
    \args 1
    \default "--:"

  with \option "-E --exclude", "Exclude a path from being scanned"
    \args "+"

  with \option "--highlight-flags", "Flags to pass to highlight"
    \args 1
    \default "-I -l --anchors -y L"

  with \option "--highlight-extra", "Set extra flags to pass to highlight"
    \args 1
    \default ""

  argl = \parse!

--
unless isDir argl.docs_dir   then makeDir argl.docs_dir
unless isDir argl.source_dir then makeDir argl.source_dir

if isFile argl.path
  generateFile argl.path, argl
else
  argl.exclude or= {}
  for node in *buildPathList argl.path
    unless contains argl.exclude, node
      generateFile node, argl
